/*
 * Copyright (C) 2011 Splitted-Desktop Systems. All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL PRECISION INSIGHT AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#define _GNU_SOURCE 1 /* RTLD_DEFAULT */
#include "config.h"
#include <dlfcn.h>
#include <assert.h>
#include "intel_driver.h"
#include "intel_compat.h"

typedef int (*drm_intel_bo_mrb_exec_func_t)(
    drm_intel_bo *bo, int used,
    drm_clip_rect_t *cliprects, int num_cliprects,
    int DR4, int ring_flag);

typedef drm_intel_bo *(*drm_intel_bo_alloc_tiled_func_t)(
    drm_intel_bufmgr *bufmgr, const char *name,
    int x, int y, int cpp, uint32_t *tiling_mode,
    unsigned long *pitch, unsigned long flags);

static int
drm_default_intel_bo_mrb_exec(drm_intel_bo *bo, int used,
                              drm_clip_rect_t * cliprects, int num_cliprects,
                              int DR4, int ring_flag)
{
    return drm_intel_bo_exec(bo, used, cliprects, num_cliprects, DR4);
}

int
drm_compat_intel_bo_mrb_exec(drm_intel_bo *bo, int used,
                             drm_clip_rect_t *cliprects, int num_cliprects,
                             int DR4, int ring_flag)
{
    static drm_intel_bo_mrb_exec_func_t func;
    if (!func) {
        func = (drm_intel_bo_mrb_exec_func_t)
            dlsym(RTLD_DEFAULT, "drm_intel_bo_mrb_exec");
        if (!func)
            func = drm_default_intel_bo_mrb_exec;
    }
    return func(bo, used, cliprects, num_cliprects, DR4, ring_flag);
}

static drm_intel_bo *
drm_default_intel_bo_alloc_tiled(drm_intel_bufmgr *bufmgr, const char *name,
                                 int x, int y, int cpp, uint32_t *tiling_mode,
                                 unsigned long *pitch, unsigned long flags)
{
    return NULL;
}

drm_intel_bo *
drm_compat_intel_bo_alloc_tiled(drm_intel_bufmgr *bufmgr, const char *name,
                                int x, int y, int cpp, uint32_t *tiling_mode,
                                unsigned long *pitch, unsigned long flags)
{
    static drm_intel_bo_alloc_tiled_func_t func;
    if (!func) {
        func = (drm_intel_bo_alloc_tiled_func_t)
            dlsym(RTLD_DEFAULT, "drm_intel_bo_alloc_tiled");
        if (!func)
            func = drm_default_intel_bo_alloc_tiled;
    }
    return func(bufmgr, name, x, y, cpp, tiling_mode, pitch, flags);
}
